==> template-engine-0.0.1.tgz: package/lib/RenderingController.js <==
var Class           = require("core").Class;
var Event           = require('core').Event.Event;
var ErrorEvent      = require('core').Event.ErrorEvent;
var EventDispatcher = require('core').Event.EventDispatcher;

var DriverManager   = require('./driver/DriverManager.js').DriverManager;
var DriverLocal     = require('./driver/DriverLocal.js').DriverLocal;
var DriverAbstract  = require('./driver/DriverAbstract.js').DriverAbstract;
var TemplateManager = require('./TemplateManager.js').TemplateManager;


/**
 * @class RenderingController Responsible for holding and parsing configuration.
 * @extends EventDispatcher
 * @requires Class 
 * @requires Event 
 * @requires ErrorEvent 
 * @requires EventDispatcher 
 * @requires DriverManager 
 * @requires DriverAbstract 
 * @requires TemplateManager
 * @param {Object} configuration Rendering configuration
 * @throws {RenderingController.Exception.WRONG_CONFIGURATION} if given configuration isn't valid - is null or is'nt {@link Object} or {@link Array}.
 */

var RenderingController = function(){
    /** @ignore */
    this.Extends = EventDispatcher;
    /**
     * @property {DriverManager} _driverManager holds reference to driverManager
     * @private
     */
    this._driverManager = null;
    /**
     * @property {TemplateManager} _templateManagers holds references to templateManagers
     * @private
     */
    this._templateManagers = [];
    /**
     * @property {Object} _configuration holds the configuration
     * @private
     */
    this._configuration = null;
    /**
     * @property {Number} _counter holds number of rendered templateManager
     * @private
     */
    this._counter = 0;

    /** @ignore */
    this.initialize = function(configuration){
        if(!configuration || typeof configuration != "object" || configuration.constructor != Array){
            throw RenderingController.Exception.WRONG_CONFIGURATION;
        }
        this._driverManager = new DriverManager();
        this._templateManagers = [];
        this._configuration = configuration;
        this._counter = 0;
    };

    /**
     * callback dispatched when templateManager renders all of its templates
     * 
     * @param {Event} e
     * @private
     */
    this._onTemplateManagerRendered = function(e){
        this._counter++;
        if(this._counter == this._templateManagers.length){
            console.log('   -- RenderingController::onTemplateManagerRendered');
            var data = [];
            for( var i = 0; i < this._templateManagers.length; i++){
                data.push(this._templateManagers[i].getTemplatesContent());
            }
            this.dispatchEvent(new Event(RenderingController.RENDERED, data));
        }
    };

    /**
     * callback dispatched when an error occurs in templateManager rendering process
     * 
     * @param {ErrorEvent} e
     * @private
     */
    this._onTemplateManagerError = function(e){
        this.dispatchEvent(new ErrorEvent(RenderingController.ERROR, e));
    };

    /**
     * wraps DriverManager.createDriver factory method,
     * creates proper instance of driver based on the given configuration
     * 
     * @param {Object} configItem Configuration of transformation rule 
     * @private
     */
    this._createDriver = function(configItem){
        var driver;

        if(configItem.hasOwnProperty('#data')){
            var data = configItem['#data'];
            if(data.hasOwnProperty('driver')){
                //TODO make a call;
                driver = DriverManager.createDriver(data.driver, data);
            }
            else{
                throw RenderingController.Exception.NO_REMOTE_DRIVER;
            }
        }else{
            driver = DriverManager.createDriver('DriverLocal', configItem.data || {});
        }

        return driver;
    };
    /**
     * Executing rendering flow. Creates {@link TemplateManager} instances, adds templates, creates drivers and fetches them
     * @public
     * @throws {RenderingController.Exception.NO_REMOTE_DRIVER} if given transformation rule has declared remote driver use <i>#data</i> but no <i>driver</i> field is declared.
     */
    this.render = function(){

        var config = this._configuration;

        // jezeli nie ma zadnych danych do renderingu zwroc blad
        if(!config.length){
            this.dispatchEvent(new ErrorEvent(RenderingController.ERROR, {}, -123, 'Empty template arguments'));
            return;
        }

        console.log('   -- RenderingController::render');
        
        // lece po wszystkich galeziach drzewa
        for( var i = 0, max = config.length; i < max; i++ ){
            // tworze TemplateManagery dla kazdej z glownych galezi drzewa
            var manager = new TemplateManager();
            this.addTemplateManager(manager);

            // lece po wszystkich itemach w danej galezi
            for(var name in config[i]){
                if(config[i].hasOwnProperty(name)){

                    //sprawdzam czy obslugujemy podany templejt
                    try{
                        var template = TemplateManager.createTemplate(name, config[i][name]);
                    }catch(e){
                        continue;
                    }

                    manager.addTemplate(template);

                    var driver = this._createDriver(config[i][name]);

                    driver.addEventListener(DriverAbstract.LOADED, template.onData, template);
                    driver.addEventListener(DriverAbstract.ERROR, template.onDataError, template);   

                    this._driverManager.addDriver(driver);
                }
            }
        }
        this._driverManager.fetch();
    };
    /**
     * Adds event listeners to the given templateManager and holds it
     * @public
     * @param {TemplateManager} manager 
     */
    this.addTemplateManager = function(manager){
        this._templateManagers.push(manager);
        manager.addEventListener(TemplateManager.RENDERED, this._onTemplateManagerRendered, this);
        manager.addEventListener(TemplateManager.ERROR, this._onTemplateManagerError, this);
        //TODO error event  
    };
};

RenderingController = new Class(new RenderingController());
/**
 * @static
 * @constant
 */
RenderingController.RENDERED = "RenderingController_RENDERED";
/**
 * @static
 * @constant
 */
RenderingController.ERROR = "RenderingController_ERROR";
/**
 * Namespace for exeptions messages.
 * @static
 * @constant
 * @namespace
 */
RenderingController.Exception = {};
/**
 * @static
 * @constant
 */
RenderingController.Exception.WRONG_CONFIGURATION = 'Given configuration is not proper';
/**
 * @static
 * @constant
 */
RenderingController.Exception.NO_REMOTE_DRIVER = "Remote data marker given but no driver specified!";

exports.RenderingController = RenderingController;

==> template-engine-0.0.5.tgz: package/lib/RenderingController.js <==
var Class           = require("core").Class;
var Event           = require('core').Event.Event;
var ErrorEvent      = require('core').Event.ErrorEvent;
var EventDispatcher = require('core').Event.EventDispatcher;

var DriverManager   = require('./driver/DriverManager.js').DriverManager;
var DriverLocal     = require('./driver/DriverLocal.js').DriverLocal;
var DriverAbstract  = require('./driver/DriverAbstract.js').DriverAbstract;
var TemplateManager = require('./TemplateManager.js').TemplateManager;


/**
 * @class RenderingController Responsible for holding and parsing configuration.
 * @extends EventDispatcher
 * @requires Class 
 * @requires Event 
 * @requires ErrorEvent 
 * @requires EventDispatcher 
 * @requires DriverManager 
 * @requires DriverAbstract 
 * @requires TemplateManager
 * @param {Object} configuration Rendering configuration
 * @throws {RenderingController.Exception.WRONG_CONFIGURATION} if given configuration isn't valid - is null or is'nt {@link Object} or {@link Array}.
 */

var RenderingController = function(){
    /** @ignore */
    this.Extends = EventDispatcher;
    /**
     * @property {DriverManager} _driverManager holds reference to driverManager
     * @private
     */
    this._driverManager = null;
    /**
     * @property {TemplateManager} _templateManagers holds references to templateManagers
     * @private
     */
    this._templateManagers = [];
    /**
     * @property {Object} _configuration holds the configuration
     * @private
     */
    this._configuration = null;
    /**
     * @property {Number} _counter holds number of rendered templateManager
     * @private
     */
    this._counter = 0;
    
    this._api = null;
    this._templatePath = null;

    /** @ignore */
    this.initialize = function(configuration, api, templatePath){
        if(!configuration || typeof configuration != "object" || configuration.constructor != Array){
            throw RenderingController.Exception.WRONG_CONFIGURATION;
        }
        this._api = api;
        this._driverManager = new DriverManager();
        this._templateManagers = [];
        this._configuration = configuration;
        this._counter = 0;
        this._templatePath = templatePath;
    };

    /**
     * callback dispatched when templateManager renders all of its templates
     * 
     * @param {Event} e
     * @private
     */
    this._onTemplateManagerRendered = function(e){
        this._counter++;
        if(this._counter == this._templateManagers.length){
            console.log('   -- RenderingController::onTemplateManagerRendered');
            var data = [];
            for( var i = 0; i < this._templateManagers.length; i++){
                data.push(this._templateManagers[i].getTemplatesContent());
            }
            this.dispatchEvent(new Event(RenderingController.RENDERED, data));
        }
    };

    /**
     * callback dispatched when an error occurs in templateManager rendering process
     * 
     * @param {ErrorEvent} e
     * @private
     */
    this._onTemplateManagerError = function(e){
        this.dispatchEvent(new ErrorEvent(RenderingController.ERROR, e));
    };

    /**
     * wraps DriverManager.createDriver factory method,
     * creates proper instance of driver based on the given configuration
     * 
     * @param {Object} configItem Configuration of transformation rule 
     * @private
     */
    this._createDriver = function(configItem){
        var driver;

        if(configItem.hasOwnProperty('#data')){
            var data = configItem['#data'];
            if(data.hasOwnProperty('driver')){
                //TODO make a call;
                driver = DriverManager.createDriver(data.driver, data);
            }
            else{
                throw RenderingController.Exception.NO_REMOTE_DRIVER;
            }
        }else{
            driver = DriverManager.createDriver('DriverLocal', configItem.data || {});
        }

        return driver;
    };
    /**
     * Executing rendering flow. Creates {@link TemplateManager} instances, adds templates, creates drivers and fetches them
     * @public
     * @throws {RenderingController.Exception.NO_REMOTE_DRIVER} if given transformation rule has declared remote driver use <i>#data</i> but no <i>driver</i> field is declared.
     */
    this.render = function(){

        var config = this._configuration;

        // jezeli nie ma zadnych danych do renderingu zwroc blad
        if(!config.length){
            this.dispatchEvent(new ErrorEvent(RenderingController.ERROR, {}, -123, 'Empty template arguments'));
            return;
        }

        console.log('   -- RenderingController::render');
        
        // lece po wszystkich galeziach drzewa
        for( var i = 0, max = config.length; i < max; i++ ){
            // tworze TemplateManagery dla kazdej z glownych galezi drzewa
            var manager = new TemplateManager();
            this.addTemplateManager(manager);

            // lece po wszystkich itemach w danej galezi
            for(var name in config[i]){
                if(config[i].hasOwnProperty(name)){

                    //sprawdzam czy obslugujemy podany templejt
                    try{
                        var template = TemplateManager.createTemplate(name, config[i][name], this._api, this._templatePath);
                    }catch(e){
                        console.log(e);
                        continue;
                    }

                    manager.addTemplate(template);

                    var driver = this._createDriver(config[i][name]);

                    driver.addEventListener(DriverAbstract.LOADED, template.onData, template);
                    driver.addEventListener(DriverAbstract.ERROR, template.onDataError, template);   

                    this._driverManager.addDriver(driver);
                }
            }
        }
        this._driverManager.fetch();
    };
    /**
     * Adds event listeners to the given templateManager and holds it
     * @public
     * @param {TemplateManager} manager 
     */
    this.addTemplateManager = function(manager){
        this._templateManagers.push(manager);
        manager.addEventListener(TemplateManager.RENDERED, this._onTemplateManagerRendered, this);
        manager.addEventListener(TemplateManager.ERROR, this._onTemplateManagerError, this);
        //TODO error event  
    };
};

RenderingController = new Class(new RenderingController());
/**
 * @static
 * @constant
 */
RenderingController.RENDERED = "RenderingController_RENDERED";
/**
 * @static
 * @constant
 */
RenderingController.ERROR = "RenderingController_ERROR";
/**
 * Namespace for exeptions messages.
 * @static
 * @constant
 * @namespace
 */
RenderingController.Exception = {};
/**
 * @static
 * @constant
 */
RenderingController.Exception.WRONG_CONFIGURATION = 'Given configuration is not proper';
/**
 * @static
 * @constant
 */
RenderingController.Exception.NO_REMOTE_DRIVER = "Remote data marker given but no driver specified!";

exports.RenderingController = RenderingController;
